==> ideaIC-213.7172.25-sources.jar: com/android/tools/idea/run/deployment/DeviceAndSnapshotComboBoxTarget.java <==
/*
 * Copyright (C) 2018 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.tools.idea.run.deployment;

import com.android.tools.idea.run.AndroidDevice;
import com.android.tools.idea.run.DeviceFutures;
import com.android.tools.idea.run.editor.DeployTarget;
import com.android.tools.idea.run.editor.DeployTargetState;
import com.google.common.annotations.VisibleForTesting;
import com.intellij.execution.Executor;
import com.intellij.execution.configurations.RunProfileState;
import com.intellij.execution.runners.ExecutionEnvironment;
import com.intellij.openapi.project.Project;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import org.jetbrains.android.facet.AndroidFacet;
import org.jetbrains.annotations.NotNull;

final class DeviceAndSnapshotComboBoxTarget implements DeployTarget {
  private final @NotNull SelectedTargetSupplier myGetSelectedTargets;
  private final @NotNull Supplier<@NotNull DeviceAndSnapshotComboBoxAction> myDeviceAndSnapshotComboBoxActionGetInstance;

  DeviceAndSnapshotComboBoxTarget(@NotNull SelectedTargetSupplier getSelectedTargets) {
    this(getSelectedTargets, DeviceAndSnapshotComboBoxAction::getInstance);
  }

  @VisibleForTesting
  DeviceAndSnapshotComboBoxTarget(@NotNull SelectedTargetSupplier getSelectedTargets,
                                  @NotNull Supplier<@NotNull DeviceAndSnapshotComboBoxAction> deviceAndSnapshotComboBoxActionGetInstance) {
    myGetSelectedTargets = getSelectedTargets;
    myDeviceAndSnapshotComboBoxActionGetInstance = deviceAndSnapshotComboBoxActionGetInstance;
  }

  @Override
  public boolean hasCustomRunProfileState(@NotNull Executor executor) {
    return false;
  }

  @NotNull
  @Override
  public RunProfileState getRunProfileState(@NotNull Executor executor,
                                            @NotNull ExecutionEnvironment environment,
                                            @NotNull DeployTargetState state) {
    throw new UnsupportedOperationException();
  }

  @NotNull
  @Override
  public DeviceFutures getDevices(@NotNull AndroidFacet facet) {
    Project project = facet.getModule().getProject();
    List<Device> devices = myDeviceAndSnapshotComboBoxActionGetInstance.get().getDevices(project).orElse(Collections.emptyList());
    Set<Target> selectedTargets = myGetSelectedTargets.get(project, devices);
    Collection<Device> selectedDevices = Target.filterDevices(selectedTargets, devices);

    bootAvailableDevices(selectedTargets, selectedDevices, project);
    return newDeviceFutures(selectedDevices);
  }

  private static void bootAvailableDevices(@NotNull Collection<@NotNull Target> selectedTargets,
                                           @NotNull Collection<@NotNull Device> selectedDevices,
                                           @NotNull Project project) {
    Map<Key, Target> map = selectedTargets.stream().collect(Collectors.toMap(Target::getDeviceKey, target -> target));

    selectedDevices.stream()
      .filter(device -> !device.isConnected())
      .map(VirtualDevice.class::cast)
      .forEach(virtualDevice -> map.get(virtualDevice.getKey()).boot(virtualDevice, project));
  }

  private static @NotNull DeviceFutures newDeviceFutures(@NotNull Collection<@NotNull Device> selectedDevices) {
    List<AndroidDevice> devices = selectedDevices.stream()
      .map(Device::getAndroidDevice)
      .collect(Collectors.toList());

    return new DeviceFutures(devices);
  }
}

==> ideaIC-2021.1.1-sources.jar: com/android/tools/idea/run/deployment/DeviceAndSnapshotComboBoxTarget.java <==
/*
 * Copyright (C) 2018 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.tools.idea.run.deployment;

import com.android.tools.idea.run.DeviceCount;
import com.android.tools.idea.run.DeviceFutures;
import com.android.tools.idea.run.deployment.DeviceAndSnapshotComboBoxTargetProvider.State;
import com.android.tools.idea.run.editor.DeployTarget;
import com.intellij.execution.Executor;
import com.intellij.execution.configurations.RunProfileState;
import com.intellij.execution.runners.ExecutionEnvironment;
import com.intellij.openapi.project.Project;
import java.util.ArrayList;
import java.util.Collection;
import org.jetbrains.android.facet.AndroidFacet;
import org.jetbrains.annotations.NotNull;

final class DeviceAndSnapshotComboBoxTarget implements DeployTarget<State> {
  @NotNull
  private final Collection<Device> myDevices;

  DeviceAndSnapshotComboBoxTarget(@NotNull Collection<Device> devices) {
    myDevices = devices;
  }

  @Override
  public boolean hasCustomRunProfileState(@NotNull Executor executor) {
    return false;
  }

  @NotNull
  @Override
  public RunProfileState getRunProfileState(@NotNull Executor executor, @NotNull ExecutionEnvironment environment, @NotNull State state) {
    throw new UnsupportedOperationException();
  }

  @NotNull
  @Override
  public DeviceFutures getDevices(@NotNull State state, @NotNull AndroidFacet facet, @NotNull DeviceCount count, boolean debug, int id) {
    DeviceFutures futures = new DeviceFutures(new ArrayList<>(myDevices.size()));
    Project project = facet.getModule().getProject();
    myDevices.forEach(device -> device.addTo(futures, project));

    return futures;
  }
}
